@@ -5,17 +5,21 @@ Data access
 from __future__ import annotations
 
 import anyio
-import datetime
 import os
 import sys
 import time
-from collections.abc import Mapping
 
-from .node import Node
-from .meta import MsgMeta
-from moat.util import NotGiven, Path, attrdict, process_args, yprint, PathLongener
+from moat.util import NotGiven, Path, PathLongener, attrdict, process_args, yprint
 from moat.util.times import ts2iso
-import contextlib
+
+from .meta import MsgMeta
+
+from collections.abc import Mapping
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from moat.lib.codec import Codec
+    from moat.link.client import Link
 
 
 def add_dates(d):
@@ -48,16 +52,16 @@ def add_dates(d):
 
 
 async def data_get(
-    conn:Link,
-    path:Path,
+    conn: Link,
+    path: Path,
     *,
-    meta:bool=False,
-    recursive:bool=True,
-    as_dict:str|None="_",
-    maxdepth:int=-1,
-    mindepth:int=0,
-    empty:bool=False,
-    raw:bool=False,
+    meta: bool = False,
+    recursive: bool = True,
+    as_dict: str | None = "_",
+    maxdepth: int = -1,
+    mindepth: int = 0,
+    empty: bool = False,
+    raw: bool = False,
     path_mangle=None,
     item_mangle=None,
     add_date=False,
@@ -86,22 +90,25 @@ async def data_get(
 
     if recursive:
         kw = {}
-        
+        a = [None, None, None]
+
         if maxdepth is not None and maxdepth >= 0:
-            kw["max_depth"] = maxdepth
+            a[2] = maxdepth
         if mindepth:
-            kw["min_depth"] = mindepth
+            a[1] = mindepth
         if empty:
             kw["empty"] = True
+        while a and a[-1] is None:
+            a.pop()
         y = {}
-        pl=PathLongener()
-        async with conn.d.walk(path, **kw).stream_in() as res:
+        pl = PathLongener()
+        async with conn.d.walk(path, *a, **kw).stream_in() as res:
             async for r in res:
-                r = await item_mangle(r)
+                r = await item_mangle(r)  # noqa:PLW2901
                 if r is None:
                     continue
-                n,p,d,*m = r
-                path=pl.long(n,p)
+                n, p, d, *m = r
+                path = pl.long(n, p)
                 path = path_mangle(path)
                 if path is None:
                     continue
@@ -110,11 +117,11 @@ async def data_get(
                     add_dates(d)
 
                 if meta:
-                    m=MsgMeta._moat__restore(m, NotGiven)
+                    m = MsgMeta._moat__restore(m, NotGiven)  # noqa: SLF001
 
                 if as_dict is not None:
                     if meta:
-                        d = dict(data=d,meta=m.repr())
+                        d = dict(data=d, meta=m.repr())
                     yy = y
                     for p in path:
                         yy = yy.setdefault(p, {})
@@ -127,10 +134,10 @@ async def data_get(
                     if raw:
                         y = path
                     elif meta:
-                        y = [path,d,m.repr()]
+                        y = [path, d, m.repr()]
                     else:
-                        y = [path,d]
-                    if isinstance(out,list):
+                        y = [path, d]
+                    if isinstance(out, list):
                         out.append(y)
                     else:
                         yprint(y, stream=out)
@@ -152,20 +159,19 @@ async def data_get(
 
                     y = simplex(y)
 
-            if isinstance(out,list):
+            if isinstance(out, list):
                 return y
             if as_dict is not None:
                 yprint(y, stream=out)
 
-            return out # end "if recursive"
+            return out  # end "if recursive"
 
-    
-    d,*m = await conn.d.get(path)
+    d, *m = await conn.d.get(path)
     if add_date:
         add_dates(d)
     if meta:
-        m=MsgMeta.restore(m)
-        d=dict(data=d,meta=m.repr())
+        m = MsgMeta.restore(m)
+        d = dict(data=d, meta=m.repr())
 
     if out is False:
         return d
@@ -179,19 +185,20 @@ async def data_get(
 
 
 async def backend_get(
-    conn:Link,
-    path:Path,
+    conn: Link,
+    path: Path,
     *,
-    meta:bool=False,
-    recursive:bool=True,
-    as_dict:str|None="_",
-    raw:bool=False,
+    meta: bool = False,
+    recursive: bool = True,
+    empty: bool = False,
+    as_dict: str | None = "_",
+    raw: bool = False,
     path_mangle=None,
     item_mangle=None,
     add_date=False,
-    codec:Codec|None=None,
+    codec: Codec | None = None,
     out=None,
-    timeout:float=0.5,
+    timeout: float = 0.5,
 ):
     """Generic code to dump a backend subtree.
 
@@ -217,8 +224,8 @@ async def backend_get(
         out = []
 
     if recursive:
-        kw = {"codec":codec}
-        
+        kw = {"codec": codec}
+
         y = {}
         async with conn.monitor(path, subtree=True, **kw) as mon:
             while True:
@@ -231,8 +238,8 @@ async def backend_get(
                 r = await item_mangle(r)
                 if r is None:
                     continue
-                p,d,m = r.topic,r.data,r.meta
-                p=p[len(path):]
+                p, d, m = r.topic, r.data, r.meta
+                p = p[len(path) :]
                 p = path_mangle(p)
                 if p is None:
                     continue
@@ -240,8 +247,8 @@ async def backend_get(
                 if add_date:
                     add_dates(d)
                 if meta:
-                    m=MsgMeta._moat__restore(m, NotGiven)
-                    d = dict(data=d,meta=m.repr())
+                    m = MsgMeta._moat__restore(m, NotGiven)  # noqa: SLF001
+                    d = dict(data=d, meta=m.repr())
 
                 if as_dict is not None:
                     yy = y
@@ -264,18 +271,17 @@ async def backend_get(
                                 y[p] = None
                             else:
                                 continue
-                    if isinstance(out,list):
+                    if isinstance(out, list):
                         out.append(y)
                     else:
                         yprint([y], stream=out)
 
-            if isinstance(out,list):
+            if isinstance(out, list):
                 return y
             yprint(y, stream=out)
 
-            return out # end "if recursive"
+            return out  # end "if recursive"
 
-    
     async with conn.monitor(path, **kw) as mon:
         r = None
         with anyio.move_on_after(timeout):
@@ -283,19 +289,19 @@ async def backend_get(
         if r is None:
             raise KeyError(path)
 
-    d,m = r.data,r.meta
+    d, m = r.data, r.meta
     if add_date:
         add_dates(d)
     if meta:
-        m=MsgMeta.restore(m)
-        d=dict(data=d,meta=m.repr())
+        m = MsgMeta.restore(m)
+        d = dict(data=d, meta=m.repr())
 
     if out is False:
         return d
     if not raw:
         yprint(d, stream=out)
     elif isinstance(d, bytes):
-        os.write(out.fileno(), res)
+        os.write(out.fileno(), d)
     else:
         out.write(str(d))
     pass  # end get
@@ -310,7 +316,7 @@ def res_get(res, attr: Path, **kw):  # pylint: disable=redefined-outer-name
     val = res.get("value", None)
     if val is None:
         return None
-    return val._get(attr, **kw)
+    return val._get(attr, **kw)  # noqa: SLF001
 
 
 def res_update(res, attr: Path, value=None, **kw):  # pylint: disable=redefined-outer-name
@@ -323,7 +329,7 @@ def res_update(res, attr: Path, value=None, **kw):  # pylint: disable=redefined-
     Returns the new value.
     """
     val = res.get("value", attrdict())
-    return val._update(attr, value=value, **kw)
+    return val._update(attr, value=value, **kw)  # noqa: SLF001
 
 
 async def node_attr(obj, path, val=NotGiven, meta=NotGiven, **kw):
@@ -340,16 +346,16 @@ async def node_attr(obj, path, val=NotGiven, meta=NotGiven, **kw):
     """
     if val is NotGiven:
         try:
-            val,*m = await obj.conn.d.get(path)
+            val, *m = await obj.conn.d.get(path)
         except KeyError:
             pass
         else:
             meta = MsgMeta.restore(m)
     val = process_args(val, **kw)
-    t = {} if meta is NotGiven else {'t':meta.timestamp}
+    t = {} if meta is NotGiven else {"t": meta.timestamp}
     if val is NotGiven:
         res = await obj.conn.d.delete(path, **t)
     else:
         res = await obj.conn.d.set(path, val, **t)
-    res = res[0],MsgMeta.restore(res[1:])
+    res = res[0], MsgMeta.restore(res[1:])
     return res
