@@ -4,26 +4,18 @@ This module describes the MoaT data link.
 
 from __future__ import annotations
 
-import os
 import anyio
-from platform import uname
 
 # pylint: disable=missing-module-docstring
 import logging
 import sys
-from functools import partial
-from pathlib import Path as FSPath
-import time
 
 import asyncclick as click
 
-from mqttproto import MQTTException
-
-from moat.util import NotGiven, P, Path, load_subgroup, yprint, attrdict
+from moat.util import NotGiven, P, Path, load_subgroup, yprint
 from moat.util.path import set_root
 
-from .backend import RawMessage, get_backend
-from .client import Link
+from .backend import get_backend
 
 try:
     from mqttproto import MQTTPublishPacket
@@ -82,17 +74,15 @@ start "moat link server" in a separate terminal, and try again.
 
 
 @load_subgroup(sub_pre="moat.link.cmd", sub_post="cli", ext_pre="moat.link", ext_post="_main.cli")
-@click.option("-n","--name", type=str, help="Name of this client (or server)")
 @click.pass_context
-async def cli(ctx,name):
+async def cli(ctx):
     """
     MoaT's data link
 
-    This collection of commands is useful for managing and building MoaT itself.
+    Commands for running and managing MoaT's data link layer.
     """
     obj = ctx.obj
     cfg = obj.cfg
-    obj.name = name
 
     if "link" not in cfg or "backend" not in cfg["link"]:
         sys.stderr.write(usage1)
@@ -110,9 +100,10 @@ async def test(obj):
     lock = anyio.Lock()
     cfg = obj.cfg.link
     set_root(cfg)
+
     async def check_root():
         try:
-            with anyio.fail_after(1) as sc:
+            with anyio.fail_after(1):
                 async with back.monitor(cfg.root) as mon:
                     async for msg in mon:
                         async with lock:
@@ -125,7 +116,7 @@ async def test(obj):
 
     async def check_server():
         try:
-            with anyio.fail_after(1) as sc:
+            with anyio.fail_after(1):
                 async with back.monitor(P(":R.run.service.main.conn")) as mon:
                     async for msg in mon:
                         async with lock:
@@ -136,9 +127,6 @@ async def test(obj):
         except TimeoutError:
             print(f"### No server link on {cfg.root}!")
 
-
-    async with get_backend(cfg) as back,back.connect(), anyio.create_task_group() as tg:
+    async with get_backend(cfg) as back, back.connect(), anyio.create_task_group() as tg:
         tg.start_soon(check_root)
         tg.start_soon(check_server)
-        
-
