@@ -1,33 +1,35 @@
 from __future__ import annotations
 
 import anyio
-import time
-import os
 import sys
+import time
 from contextlib import asynccontextmanager, nullcontext
-from tempfile import TemporaryDirectory
-from pathlib import Path as FSPath
 from functools import partial
+from pathlib import Path as FSPath
+from tempfile import TemporaryDirectory
 
-from moat.link.client import Link
-from moat.link.server import Server
-from moat.link.backend import get_backend
 from moat.util import (  # pylint:disable=no-name-in-module
     CFG,  # noqa:F401
-    ensure_cfg,
     CtxObj,
-    attrdict,
-    combine_dict,
     NotGiven,
-    Path,P,
     Root,
     ValueEvent,
+    attrdict,
+    combine_dict,
+    ensure_cfg,
 )
+from moat.link.backend import get_backend
+from moat.link.client import Link
+from moat.link.server import Server
 
 from typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
-    from typing import Any, AsyncIterator,Self
+    from moat.link.backend import Backend
+    from moat.link.client import LinkCommon
+
+    from collections.abc import AsyncIterator
+    from typing import Never, Self
 
 ensure_cfg("moat.link")
 
@@ -46,32 +48,46 @@ async def run_broker(cfg, *, task_status):
 
     The task status returns the port we're listening on.
     """
-    #cfg  # noqa:B018  # pyright:ignore
-    #broker = AsyncMQTTBroker(("127.0.0.1", 0))
-    #await broker.serve(task_status=task_status)
-    port=40000+(os.getpid()+123)%10000
+    cfg  # pyright:ignore # noqa:B018
+    # broker = AsyncMQTTBroker(("127.0.0.1", 0))
+    # await broker.serve(task_status=task_status)
+    from anyio.pytest_plugin import FreePortFactory  # noqa: PLC0415
+    from socket import SOCK_STREAM  # noqa: PLC0415
+
+    port = FreePortFactory(SOCK_STREAM)()
+
     async with (
-            anyio.NamedTemporaryFile(mode="w+") as tf,
-            anyio.create_task_group() as tg,
-            ):
-        await tf.write(f"""\
+        anyio.TemporaryDirectory() as td,
+        anyio.create_task_group() as tg,
+    ):
+        tf = anyio.Path(td) / "config"
+        await tf.write_text(
+            f"""\
 allow_anonymous true
 retained_messages_mode enabled_without_persistence
 thread_count 1
 
 listen {{
     protocol mqtt
-    port { port }
+    port {port}
     inet4_bind_address 127.0.0.1
 }}
-""")
-        await tf.flush()
+"""
+        )
 
-        tg.start_soon(partial(anyio.run_process,["flashmq","-c",tf.name],stderr=sys.stderr,stdout=sys.stdout))
+        tg.start_soon(
+            partial(
+                anyio.run_process,
+                ["flashmq", "-c", str(tf)],
+                stderr=sys.stderr,
+                stdout=sys.stdout,
+                env=dict(HOME=td),
+            )
+        )
         for _ in range(20):
             try:
-                sock = await anyio.connect_tcp("127.0.0.1",port)
-            except EnvironmentError:
+                sock = await anyio.connect_tcp("127.0.0.1", port)
+            except OSError:
                 await anyio.sleep(0.1)
             else:
                 await sock.aclose()
@@ -86,7 +102,8 @@ class Scaffold(CtxObj):
     """
     Basic testcase runner for testing with an ephemeral MQTT server.
     """
-    tempdir:str|None
+
+    tempdir: str | None
 
     def __init__(self, cfg: attrdict, use_servers=True, tempdir: str | None = None):
         ensure_cfg("moat.link.server", cfg)
@@ -97,10 +114,10 @@ class Scaffold(CtxObj):
         self.cfg.backend.setdefault("codec", "std-cbor")
         self.cfg.backend.setdefault("keep_alive", 9999)
 
-        self.cfg.server.ping.cycle=.5
-        self.cfg.server.ping.gap=.15
-        self.cfg.server.ping.override=True
-        self.cfg.server.timeout.startup=3
+        self.cfg.server.ping.cycle = 0.5
+        self.cfg.server.ping.gap = 0.15
+        self.cfg.server.ping.override = True
+        self.cfg.server.timeout.startup = 3
 
         self._tempdir = tempdir
 
@@ -148,7 +165,7 @@ class Scaffold(CtxObj):
         async with self.backend_(cfg, **kw) as bk:
             task_status.started(bk)
             await anyio.sleep_forever()
-            assert False  # noqa:B011,PT015
+            raise AssertionError
 
     async def server(self, cfg: dict | None = None, **kw) -> tuple[Server, list[dict]]:
         """
@@ -168,19 +185,19 @@ class Scaffold(CtxObj):
         cfg["server"]["port"] = 0
         if self.tempdir is not None:
             cfg["server"]["save"]["dir"] = self.tempdir / "data"
-        global _seq  # noqa:PLW0603
+        global _seq
         _seq += 1
 
         s = Server(cfg, f"S_{_seq}", **kw)
         await s.serve(task_status=task_status)
 
     @asynccontextmanager
-    async def server_(self, cfg:dict|None=None, **kw) -> None:
+    async def server_(self, cfg: dict | None = None, **kw) -> None:
         """
         Runs a basic MoaT-Link server. (async context manager)
         """
         async with anyio.create_task_group() as tg:
-            yield await tg.start(self._run_server,cfg,kw)
+            yield await tg.start(self._run_server, cfg, kw)
             tg.cancel_scope.cancel()
 
     async def client(self, *a, **kw):
@@ -194,56 +211,65 @@ class Scaffold(CtxObj):
         async with self.client_(*a, **kw) as li:
             task_status.started(li)
             await anyio.sleep_forever()
-            assert False  # noqa:B011,PT015
+            raise AssertionError
 
     @asynccontextmanager
-    async def client_(self, cfg:dict|None=None, cli:LinkCommon|None=None, name=None) -> Never:
+    async def client_(
+        self, cfg: dict | None = None, cli: LinkCommon | None = None, name=None
+    ) -> Never:
         """
         Start a client (async context manager)
         """
         if cli is None:
             cfg = combine_dict(cfg, self.cfg, cls=attrdict) if cfg else self.cfg
 
-            global _seq  # noqa:PLW0603
+            global _seq
             _seq += 1
             name = f"C_{_seq}"
 
-            cli = Link(cfg,name)
+            cli = Link(cfg, name)
 
         async with cli as li:
             yield li
 
     @asynccontextmanager
-    async def do_watch(self, path, exp=NotGiven, n=0, *a, **kw):
+    async def do_watch(self, path, exp=NotGiven, n=0, *a, **kw) -> AsyncIterator[ValueEvent]:
         """
-        Run a client that expects @xp and appends all non-exp
+        Run a client that expects @exp and appends all non-exp
         results to @rd.
 
         All other args+kw are forwarded to `Link.d_watch`.
+
+        The results are appended to a list that's posted to the event when
+        `do_watch` ends (for whatever reason).
         """
         async with anyio.create_task_group() as tg, self.client_() as c:
             evt = ValueEvent()
-            got=0
+
             @tg.start_soon
             async def work():
-                res=[]
+                res = []
                 try:
-                    async with c.d_watch(path,*a,**kw) as mon:
+                    async with c.d_watch(path, *a, **kw) as mon:
                         async for r in mon:
-                            if not kw.get("meta") and not kw.get("subtree"):
-                                r = (r,)
                             if kw.get("meta"):
                                 t = time.time()
                                 assert t - 1 < r[-1].timestamp < t
-                            if exp is not NotGiven and (not kw.get("subtree") or not len(r[0])) and r[kw.get("subtree",0)] == exp:
+                            elif not kw.get("subtree"):
+                                # neither meta nor subtree
+                                r = (r,)  # noqa:PLW2901
+                            if (
+                                exp is not NotGiven
+                                and (not kw.get("subtree") or not len(r[0]))
+                                and r[kw.get("subtree", 0)] == exp
+                            ):
+                                # Skip if toplevel match
                                 return
                             res.append(r)
-                            if len(res)==n:
+                            if len(res) == n:
                                 return
                 finally:
                     evt.set(res)
+
             yield evt
             tg.cancel_scope.cancel()
-
-
-
