@@ -13,19 +13,21 @@ from contextlib import asynccontextmanager
 
 from attrs import define, field
 
-from moat.util import CtxObj, NotGiven, Path, Root, RootPath, attrdict, get_codec
+from moat.util import CtxObj, NotGiven, Path, Root, attrdict, get_codec
 
 from typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
+    from mqttproto import QoS
+
     from moat.lib.codec import Codec
     from moat.link.meta import MsgMeta
 
-    from typing import Any, Self, ClassVar
     from collections.abc import AsyncIterator
+    from typing import Any, ClassVar, Literal, Self
 
 
-__all__ = ["get_backend", "get_codec", "Backend", "Message", "RawMessage"]
+__all__ = ["Backend", "Message", "RawMessage", "get_backend", "get_codec"]
 
 
 @define
@@ -37,8 +39,8 @@ class Message[TData]:
     topic: Path = field()
     data: TData = field()
     meta: MsgMeta = field()
-    prop: dict[str,Any] = field()
-    retain: bool|None = field(default=False)
+    prop: dict[str, Any] = field(repr=False)
+    retain: bool | None = field(default=False)
 
     raw: ClassVar[bool] = False
 
@@ -61,7 +63,7 @@ class Backend(CtxObj, metaclass=ABCMeta):
     _njobs: int = 0
     _ended: anyio.Event | None = None
 
-    def __init__(self, cfg: attrdict, name: str, id:str):
+    def __init__(self, cfg: attrdict, name: str, id: str):
         self.cfg = cfg
         self.name = name
         self.id = id
@@ -123,7 +125,7 @@ class Backend(CtxObj, metaclass=ABCMeta):
         """
         Send a somewhat-free-form error message.
         """
-        if not isinstance(subpath[0], RootPath):
+        if subpath.prefix is None:
             subpath = Root.get() / "error" + subpath
         if msg:
             kw["msg"] = msg
@@ -145,7 +147,7 @@ def get_backend(cfg: dict, **kw) -> Backend:
     """
     Fetch the backend named in the config and initialize it.
     """
-    from importlib import import_module
+    from importlib import import_module  # noqa: PLC0415
 
     cfg = cfg["backend"]
     name = cfg["driver"]
