@@ -6,54 +6,77 @@ from __future__ import annotations
 
 import anyio
 import logging
-import time
-import sys
 import os
-from contextlib import asynccontextmanager, suppress, nullcontext, aclosing
-from traceback import format_exception
-from platform import uname
+import platform
+import time
+from contextlib import asynccontextmanager, nullcontext, suppress
+from contextvars import ContextVar
 from functools import partial
+from traceback import format_exception
 
 import outcome
-from attrs import define,field
+from attrs import define, field
+from mqttproto import QoS, RetainHandling
 
-from mqttproto import RetainHandling, QoS
-from moat.lib.cmd.base import MsgSender, Caller
-from moat.util import CtxObj, P, Root, ValueEvent, timed_ctx, gen_ident, ungroup, NotGiven,Path, PathLongener,srepr, attrdict
-from moat.util.compat import CancelledError
+from moat.util import (
+    CtxObj,
+    NotGiven,
+    P,
+    Path,
+    PathLongener,
+    Root,
+    ValueEvent,
+    attrdict,
+    ctx_as,
+    gen_ident,
+    srepr,
+    timed_ctx,
+    ungroup,
+)
+from moat.lib.cmd.base import Caller, MsgSender
 from moat.util.random import al_unique
 
+from .auth import AnonAuth, TokenAuth
 from .common import CmdCommon
 from .conn import TCPConn, UnixConn
-from .auth import AnonAuth, TokenAuth
+from .exceptions import AuthError, ClientCancelledError
 from .hello import Hello
 from .meta import MsgMeta
 from .node import Node
 
-from typing import TYPE_CHECKING,overload
+from typing import TYPE_CHECKING, overload
+
 if TYPE_CHECKING:
-    from typing import Any
+    from contextlib import AbstractAsyncContextManager
+
+    from moat.lib.cmd.base import MsgHandler
+    from moat.lib.cmd.msg import Msg
     from moat.link.node.codec import CodecNode
 
+    from .backend import Message
+    from .schema import Data
+    from .schema import SchemaName as S
+
+    from collections.abc import AsyncIterator, Awaitable
+    from typing import Any, Literal
+
+
 class _Requeue(Exception):
     pass
 
 
-from typing import TYPE_CHECKING
+__all__ = ["BasicLink", "Link", "LinkCommon", "get_link"]
 
-if TYPE_CHECKING:
-    from collections.abc import Awaitable
-    from typing import AsyncContextManager,AsyncIterator
+_the_link = ContextVar("_the_link")
 
-    from moat.lib.cmd.base import MsgHandler
-    from moat.lib.cmd.msg import Msg
-
-    from .schema import Data
-    from .schema import SchemaName as S
-    from .backend import Message
+local_addrs = {"localhost", "127.0.0.1", "::1"}
 
 
-__all__ = ["Link", "LinkCommon", "BasicLink"]
+def get_link() -> Link | None:
+    """
+    Return the global Link instance, if one exists.
+    """
+    return _the_link.get(None)
 
 
 class TS(anyio.abc.TaskStatus):
@@ -106,21 +129,24 @@ class BasicCmd:
 
 class LinkCommon(CmdCommon):
     """
-    Common code for links to MQTT only (BaseLink) and a MoaT-Link server (Link).
+    Common code for links to MQTT only (BasicLink) and to a MoaT-Link server (Link).
     """
-    protocol_version:int = -1
-    name:str
-    server_name:str
 
-    def __init__(self, cfg, name: str | None = None, is_server:bool = False):
+    protocol_version: int = -1
+    name: str
+    server_name: str
+    is_server: bool = False
+
+    def __init__(self, cfg, name: str | None = None):
         self.cfg = cfg
         self._id = gen_ident(12, alphabet=al_unique)
-        if name is None:
+        if name is not None:
+            self.is_server = True
+        else:
             name = cfg.get("client_id")
-        if name is None:
-            name = "_" + self._id
+            if name is None:
+                name = "_" + self._id
         self.name = name
-        self.is_server = is_server
 
         self._cmdq_w, self._cmdq_r = anyio.create_memory_object_stream(5)
         self.logger = logging.getLogger(f"moat.link.client.{name}")
@@ -137,7 +163,7 @@ class LinkCommon(CmdCommon):
         return super().handle(msg, rpath, *add)
 
     @property
-    def id(self):
+    def id(self):  # noqa:D102
         return self._id
 
     async def _connected_port(self, *, task_status=anyio.TASK_STATUS_IGNORED):
@@ -146,12 +172,14 @@ class LinkCommon(CmdCommon):
             await anyio.sleep_forever()
 
     @asynccontextmanager
-    async def _connect_one(self, remote:dict|str, data: dict|None=None) -> MsgHandler:
+    async def _connect_one(self, remote: dict | str, data: dict | None = None) -> MsgHandler:
         auth_out = []
-        if isinstance(remote,dict):
+        if isinstance(remote, dict):
             with suppress(KeyError):
                 auth_out.append(TokenAuth(data["auth"]["token"]))
-            conn_ = TCPConn(self, remote_host=remote["host"], remote_port=remote["port"])
+            conn_ = TCPConn(
+                self, remote_host=remote["host"], remote_port=remote["port"], logger=self.logger
+            )
         else:
             conn_ = UnixConn(self, path=remote, logger=self.logger.debug)
 
@@ -161,13 +189,13 @@ class LinkCommon(CmdCommon):
 
         async with conn_ as conn:
             if self.logger.isEnabledFor(logging.INFO):
-                print(f"Connection {self.name} to {srepr(remote)}", file=sys.stderr)
+                self.logger.info("Connection %s to %s", self.name, srepr(remote))
             handler = MsgSender(conn)
             if (res := await self._hello.run(handler)) is False:
                 raise AuthError("Initial handshake failed")
 
-            if isinstance(res,dict):
-                name = res.pop("name",None)
+            if isinstance(res, dict):
+                name = res.pop("name", None)
                 if name is not None:
                     self.name = name
                 if res:
@@ -185,10 +213,10 @@ class LinkCommon(CmdCommon):
             raise EOFError
 
 
-class ClientCaller(Caller):
-    def __init__(self,sender,*a,**kw):
-        self._link = sender._link
-        super().__init__(sender,*a,**kw)
+class ClientCaller(Caller):  # nqa:D102
+    def __init__(self, sender, *a, **kw):
+        self._link = sender._link  # noqa:SLF001
+        super().__init__(sender, *a, **kw)
 
     @asynccontextmanager
     async def _ctx(self):
@@ -200,10 +228,10 @@ class ClientCaller(Caller):
         "helper for __await__ that calls the remote handler"
         link = await self._link.get_link()
         cmd, a, kw = self.data
-        return await link.root._sender.cmd(cmd, *a, **kw)
+        return await link.root._sender.cmd(cmd, *a, **kw)  # noqa:SLF001
 
 
-class _Sender(MsgSender):
+class LinkSender(MsgSender):
     """
     This is the client-side front-end to a "standard" MoaT-Link connection.
 
@@ -211,17 +239,22 @@ class _Sender(MsgSender):
     the corresponding backend commands, with an underscore replacing the
     dot.
     """
+
     Caller_ = ClientCaller
 
-    _codec_tree:CodecNode|None = None
+    _codec_tree: CodecNode | None = None
 
-    def __init__(self, link:LinkCommon):
+    def __init__(self, link: LinkCommon):
         self._link = link
 
     @property
     def root(self):
         return self
 
+    @property
+    def link(self):
+        return self._link
+
     @property
     def name(self) -> str:
         return self._link.name
@@ -241,47 +274,63 @@ class _Sender(MsgSender):
         srv = await self._link.get_link()
         await srv.handle(msg, rcmd)
 
-    def find_handler(self, path, may_stream: bool = False) -> tuple[MsgHandler, Path]:
+    def find_handler(self, path, cmd: bool = False) -> tuple[MsgHandler, Path]:
         """
         Standard sub-dispatcher redirector, no-op.
         """
+        cmd  # noqa:B018
         return self, path
 
     @overload
-    def d_get(self, path:Path, meta:Literal[True]) -> tuple[Any,MsgMeta]:
-        ...
+    def d_get(self, path: Path, meta: Literal[True]) -> tuple[Any, MsgMeta]: ...
 
     @overload
-    def d_get(self, path:Path) -> Any:
-        ...
+    def d_get(self, path: Path) -> Any: ...
 
-    async def d_get(self, path:Path, meta:bool=False) -> tuple[Any,MsgMeta]:
+    async def d_get(self, path: Path, meta: bool = False) -> tuple[Any, MsgMeta]:
         """
         Data retrieval. Calls the server's ``d.get`` method.
 
         Returns a data+metadata tuple if @meta is True, otherwise just the
         data.
         """
-        if len(path) and isinstance(path[0],Path):
+        if len(path) and isinstance(path[0], Path):
             raise ValueError("Don't use a root-prefixed path here.")
 
         res = await self.d.get(path)
         if not meta:
             return res[0]
-        return res[0],MsgMeta.restore(res[1:])
+        return res[0], MsgMeta.restore(res[1:])
 
     @overload
-    def d_set(self, path:Path, t:float|None=None, meta:Literal[True]=True) -> tuple[Any,MsgMeta]:
-        ...
+    async def d_set(
+        self,
+        path: Path,
+        data: Any = NotGiven,
+        t: float | None = None,
+        meta: Literal[True] = True,
+        retain: bool | None = None,
+    ) -> tuple[Any, MsgMeta]: ...
 
     @overload
-    def d_set(self, path:Path, t:float|None=None) -> None:
-        ...
+    async def d_set(
+        self,
+        path: Path,
+        data: Any = NotGiven,
+        t: float | None = None,
+        meta: Literal[False] = False,
+        retain: bool | None = None,
+    ) -> None: ...
 
-    async def d_set(self, path:Path, data:Any=NotGiven,
-                    meta:MsgMeta|None=None, t:float|None=None,
-                    with_prev:bool=False, retain:bool|None=None,
-                   ) -> None|tuple[Any,MsgMeta]:
+    async def d_set(
+        self,
+        path: Path,
+        data: Any = NotGiven,
+        meta: MsgMeta | None = None,
+        t: float | None = None,
+        with_prev: bool = False,
+        retain: bool | None = None,
+    ) -> None | tuple[Any, MsgMeta]:
         """
         Data update.
 
@@ -289,27 +338,36 @@ class _Sender(MsgSender):
         requested via @with_prev, goes through the server. Otherwise posts to
         MQTT directly.
         """
-        if path and isinstance(path[0],Path):
+        if path and isinstance(path[0], Path):
             raise ValueError("Don't use a root-prefixed path here.")
 
         if meta is None:
-            meta=MsgMeta(origin=self._link.name)
+            meta = MsgMeta(origin=self._link.name)
         if t is None and not with_prev:
-            await self.send(Root.get()+path, data=data, meta=meta, retain=len(path)==0 or path[0]!="run")
+            if retain is None:
+                retain = len(path) == 0 or path[0] != "run"
+            await self.send(Root.get() + path, data=data, meta=meta, retain=retain)
             return
-        tt = {} if t is None else {"t":t}
+        tt = {} if t is None else {"t": t}
         res = await self.d.set(path, data, meta, **tt)
         if not with_prev:
             return res[0]
-        meta = MsgMeta.restore(res[1:]) if len(res)>1 else None
-        return res[0],meta
+        meta = MsgMeta.restore(res[1:]) if len(res) > 1 else None
+        return res[0], meta
 
     @asynccontextmanager
-    async def d_walk(self, path:Path, meta:bool=False, min_ts:float=0, min_depth:int=0, max_depth:int=255) -> AsyncIterator[tuple[str,Any,MsgMeta]]:
+    async def d_walk(
+        self,
+        path: Path,
+        meta: bool = False,
+        min_ts: float = 0,
+        min_depth: int = 0,
+        max_depth: int = 255,
+    ) -> AsyncIterator[tuple[str, Any, MsgMeta]]:
         """
         Fetch a (limited) subtree.
         """
-        args = [path,min_ts,min_depth,max_depth]
+        args = [path, min_ts, min_depth, max_depth]
         if args[-1] == 255:
             args.pop()
             if args[-1] == 0:
@@ -318,39 +376,91 @@ class _Sender(MsgSender):
                     args.pop()
 
         async with self.d.walk(*args).stream_in() as mon:
-            yield Walker(mon,meta=meta)
-
+            yield Walker(mon, meta=meta)
 
     # No use marking as Any|None if @mark is set
     @overload
-    def d_watch(self, path:Path, mark:bool, meta:Literal[False],subtree:Literal[False],state:bool|None=None) -> AsyncContextManager[AsyncIterator[Any]]:
-        ...
+    def d_watch(
+        self,
+        path: Path,
+        mark: bool = False,
+        meta: Literal[False] = False,
+        subtree: Literal[False] = False,
+        state: bool | Literal[NotGiven] | None = None,
+    ) -> AbstractAsyncContextManager[AsyncIterator[Any]]: ...
 
     @overload
-    def d_watch(self, path:Path, mark:Literal[False], meta:Literal[True],subtree:Literal[False],state:bool|None=None) -> AsyncContextManager[AsyncIterator[tuple[Any,MsgMeta]]]:
-        ...
+    def d_watch(
+        self,
+        path: Path,
+        mark: Literal[False] = False,
+        meta: Literal[True] = True,
+        subtree: Literal[False] = False,
+        state: bool | Literal[NotGiven] | None = None,
+    ) -> AbstractAsyncContextManager[AsyncIterator[tuple[Any, MsgMeta]]]: ...
 
     @overload
-    def d_watch(self, path:Path, mark:Literal[False], meta:Literal[True],subtree:Literal[True],state:bool|None=None) -> AsyncContextManager[AsyncIterator[tuple[Path,Any,MsgMeta]]]:
-        ...
+    def d_watch(
+        self,
+        path: Path,
+        mark: Literal[False],
+        meta: Literal[True],
+        subtree: Literal[True],
+        state: bool | Literal[NotGiven] | None = None,
+    ) -> AbstractAsyncContextManager[AsyncIterator[tuple[Path, Any, MsgMeta]]]: ...
 
     @overload
-    def d_watch(self, path:Path, mark:Literal[False], meta:Literal[False],subtree:Literal[True],state:bool|None=None) -> AsyncContextManager[AsyncIterator[tuple[Path,Any]]]:
-        ...
+    def d_watch(
+        self,
+        path: Path,
+        mark: Literal[False] = False,
+        meta: Literal[False] = False,
+        subtree: Literal[True] = True,
+        state: bool | Literal[NotGiven] | None = None,
+    ) -> AbstractAsyncContextManager[AsyncIterator[tuple[Path, Any]]]: ...
 
     @overload
-    def d_watch(self, path:Path, mark:Literal[True], meta:Literal[True],subtree:Literal[False],state:bool|None=None) -> AsyncContextManager[AsyncIterator[None|tuple[Any,MsgMeta]]]:
-        ...
+    def d_watch(
+        self,
+        path: Path,
+        mark: Literal[True],
+        meta: Literal[True],
+        subtree: Literal[False] = False,
+        state: bool | Literal[NotGiven] | None = None,
+    ) -> AbstractAsyncContextManager[AsyncIterator[None | tuple[Any, MsgMeta]]]: ...
 
     @overload
-    def d_watch(self, path:Path, mark:Literal[True], meta:Literal[True],subtree:Literal[True],state:bool|None=None) -> AsyncContextManager[AsyncIterator[None|tuple[Path,Any,MsgMeta]]]:
-        ...
+    def d_watch(
+        self,
+        path: Path,
+        mark: Literal[True],
+        meta: Literal[True],
+        subtree: Literal[True],
+        state: bool | Literal[NotGiven] | None = None,
+    ) -> AbstractAsyncContextManager[AsyncIterator[None | tuple[Path, Any, MsgMeta]]]: ...
 
     @overload
-    def d_watch(self, path:Path, mark:Literal[True], meta:Literal[False],subtree:Literal[True],state:bool|None=None) -> AsyncContextManager[AsyncIterator[None|tuple[Path,Any]]]:
-        ...
+    def d_watch(
+        self,
+        path: Path,
+        mark: Literal[True],
+        meta: Literal[False] = False,
+        subtree: Literal[True] = True,
+        state: bool | Literal[NotGiven] | None = None,
+    ) -> AbstractAsyncContextManager[AsyncIterator[None | tuple[Path, Any]]]: ...
 
-    def d_watch(self, path:Path, meta:bool=False, subtree:bool=False, state:bool|None=None, max_age:float|None=None, mark:bool=False, cls:type=Node) -> AsyncContextManager[AsyncIterator[tuple[Path,Any,MsgMeta]]]:
+    def d_watch(
+        self,
+        path: Path,
+        meta: bool = False,
+        subtree: bool = False,
+        state: bool | None | NotGiven = None,
+        age: float | None = None,
+        mark: bool = False,
+        min_length: int | None = None,
+        max_length: int | None = None,
+        cls: type = Node,
+    ) -> AbstractAsyncContextManager[AsyncIterator[tuple[Path, Any, MsgMeta]]]:
         """
         Monitor a node or subtree.
 
@@ -358,37 +468,39 @@ class _Sender(MsgSender):
         @meta: flag whether to return metadata too
         @subtree: flag whether to watch a subtree, not just this node
         @mark: yield `None` when the initial state has been transmitted
-        @state: send the current state (True), updates (False) or both (None)
+        @state: send the current state (True), updates (False), both with
+                current data from the server (None), or both via MQTT (NotGiven).
+        @age: cutoff this many seconds ago. Older entries are skipped.
         @cls: type of root node (default `Node`)
 
         This method returns an async context manager which yields an async iterator.
         The iterator yields node data if neither @subtree nor @meta is set.
         Otherwise it yields tuples. The first item is the path if @subtree
         is set; the last item is the metadata if @meta is set.
+
+        ``state=NotGiven`` does not work with @mark=True, or when the same
+        subscription is already active somewhere else.
         """
-        return Watcher(self, path, meta, subtree, state, max_age, mark, cls)
+        return Watcher(self, path, meta, subtree, state, age, mark, cls, min_length, max_length)
 
-
-    async def e_exc(self, path:Path, exc:Exception, **kw):
+    async def e_exc(self, path: Path, exc: Exception, **kw):
         """
         Report an exception, with backtrace.
         """
         exc = ungroup.one(exc)
 
-        kw["_bt"]=format_exception(exc)
-        kw["_exc"]=exc
+        kw["_bt"] = format_exception(exc)
+        kw["_exc"] = exc
 
-        await self.e.exc(path,repr(exc),**kw)
+        await self.e.exc(path, repr(exc), **kw)
 
-
-    async def e_info(self, path:Path, txt:str, **kw):
+    async def e_info(self, path: Path, txt: str, **kw):
         """
         Report a problem that's not an exception.
         """
-        await self.e.info(path,txt,**kw)
+        await self.e.info(path, txt, **kw)
 
-
-    async def e_ack(self, path:Path, **kw):
+    async def e_ack(self, path: Path, **kw):
         """
         Report that $USER has been notified of a problem.
 
@@ -396,17 +508,16 @@ class _Sender(MsgSender):
         time it happens), or a timestamp (notify if it happens again after
         $time).
         """
-        await self.e.ack(path,**kw)
+        await self.e.ack(path, **kw)
 
-
-    async def e_ok(self, path:Path, **kw):
+    async def e_ok(self, path: Path, **kw):
         """
         Report that something is working.
         """
-        await self.e.ok(path,**kw)
+        await self.e.ok(path, **kw)
 
     @asynccontextmanager
-    async def e_wrap(self, path:Path, **kw):
+    async def e_wrap(self, path: Path, **kw):
         """
         An error-handling context manager. It opens a stream to the server
         and reports either success or failure over it.
@@ -420,8 +531,7 @@ class _Sender(MsgSender):
             yield mon
             await mon.send(True)
 
-
-    def monitor(self, path:Path, *a, **kw) -> AsyncContextManager[AsyncIterator[Message]]:
+    def monitor(self, path: Path, *a, **kw) -> AbstractAsyncContextManager[AsyncIterator[Message]]:
         """
         Watch this path.
 
@@ -432,7 +542,7 @@ class _Sender(MsgSender):
         """
         return self._link.backend.monitor(path, *a, **kw)
 
-    def send(self, path:Path, *a, **kw) -> Awaitable[None]:
+    def send(self, path: Path, *a, **kw) -> Awaitable[None]:
         """
         Send to this path.
 
@@ -461,9 +571,15 @@ class _Sender(MsgSender):
         mostly-accurate time.
         """
         (st,) = await self.cmd(P("i.stamp"))
-        await self.send(P(":R.run.service.main.stamp")/self._link.server_name, st, retain=False)
+        await self.send(P(":R.run.service.main.stamp") / self._link.server_name, st, retain=False)
         await self.cmd(P("i.sync"), st)
 
+    async def i_checkid(self, id: str) -> bool:
+        """
+        Check whether the given client ID is still valid.
+        """
+        (st,) = await self.cmd(P("i.checkid"), id)
+        return st
 
     async def _get_tree(self, path, *, task_status, **kw):
         async with self.d_watch(path, **kw) as w:
@@ -478,19 +594,30 @@ class _Sender(MsgSender):
         if self._codec_tree is None:
             self._codec_tree = evt = anyio.Event()
 
-            from moat.link.node.codec import CodecNode
-            self._codec_tree = await self._link.tg.start(partial(self._get_tree, P("codec"), subtree=True, state=None,meta=False, cls=CodecNode))
+            from moat.link.node.codec import CodecNode  # noqa: PLC0415
+
+            self._codec_tree = await self._link.tg.start(
+                partial(
+                    self._get_tree, P("codec"), subtree=True, state=None, meta=False, cls=CodecNode
+                )
+            )
             evt.set()
 
-        elif isinstance(self._codec_tree,anyio.abc.Event):
+        elif isinstance(self._codec_tree, anyio.abc.Event):
             await self._codec_tree.wait()
 
         return self._codec_tree
 
+    async def get_service(self, srv: Path) -> MsgHandler:
+        res = await self.d_get(P("run.host") + srv)
+        return self.sub_at(P("cl") / res["id"] + res["path"])
+
 
 class Link(LinkCommon, CtxObj):
     """
     This class combines the back-end link with a connection to a MoaT-Link server.
+
+    If "common" is set, create global state.
     """
 
     _server: ValueEvent = None
@@ -500,18 +627,28 @@ class Link(LinkCommon, CtxObj):
     _server_up: anyio.Event
     _last_link: Msg | None = None
     _last_link_seen: anyio.Event
-    _port: str|None=None
-    _state: str="init"
+    _port: str | None = None
+    _state: str = "init"
+    _common: bool = False
 
-    def __init__(self, cfg, name: str | None = None):
+    def __new__(cls, cfg, name: str | None = None, common: bool = False):  # noqa:D102
+        cfg, name  # noqa:B018
+        if common and (link := _the_link.get(NotGiven)) is not NotGiven:
+            return link
+        return super().__new__(cls)
+
+    def __init__(self, cfg, name: str | None = None, common: bool = False):
+        if _the_link.get(NotGiven) is self:
+            return
         super().__init__(cfg, name=name)
         self._retry_msgs: set[BasicCmd] = set()
         self._server_up = anyio.Event()
         self._state_change = anyio.Event()
+        self._common = common
         with suppress(AttributeError):
             self._port = self.cfg.client.port
 
-    async def set_state(self, state:str):
+    async def set_state(self, state: str):
         """
         Set the state string for our ping message
         """
@@ -529,77 +666,115 @@ class Link(LinkCommon, CtxObj):
 
     @property
     def _ping_path(self):
-        return P("run.ping.id")/self._id
+        return P("run.ping.id") / self._id
 
-    async def _ping(self, *, task_status=anyio.TASK_STATUS_IGNORED):
-        path = Root.get()+self._ping_path
+    @property
+    def _id_path(self):
+        return P("run.id") / self._id
+
+    async def _send_id(self):
+        await self.sdr.d_set(
+            self._id_path,
+            data=dict(
+                host=platform.node(),
+                pid=os.getpid(),
+            ),
+            retain=True,
+        )
+
+    async def _send_ping(self, *, task_status=anyio.TASK_STATUS_IGNORED):
+        """
+        Periodially (or when our state changes) send a Ping.
+
+        This also sends our ID message, after the first ping,
+        but only if the task status isn't ignored.
+        """
+        path = Root.get() + self._ping_path
         while True:
-            await self.backend.send(path, data=dict(
-                up=True,
-                state=self._state,
-            ), retain=False, meta=False)
+            await self.backend.send(
+                path,
+                data=dict(
+                    up=True,
+                    state=self._state,
+                ),
+                retain=False,
+                meta=False,
+            )
 
             if task_status is not anyio.TASK_STATUS_IGNORED:
-                await self.sdr.d_set(P("run.id")/self.id,data=dict(
-                    host=uname().node,
-                    pid=os.getpid(),
-                ), retain=True)
+                # send initial run.id message, *after* the ping
+                await self._send_id()
                 task_status.started()
-                task_status=anyio.TASK_STATUS_IGNORED
+                task_status = anyio.TASK_STATUS_IGNORED
 
             if self._state == "init":
                 self._state = "auto"
 
-            with anyio.move_on_after(self.cfg.timeout.ping):
+            with anyio.move_on_after(self.cfg.timeout.ping.every):
                 await self._state_change.wait()
                 self._state_change = anyio.Event()
 
+    async def _monitor_id(self, *, task_status=anyio.TASK_STATUS_IGNORED):
+        path = Root.get() + self._id_path
+        async with self.backend.monitor(path) as mon:
+            task_status.started()
+            async for msg in mon:
+                if msg.data is NotGiven:
+                    raise ClientCancelledError(msg.meta)
+
     @asynccontextmanager
     async def _ctx(self):
-        from .backend import get_backend
+        from .backend import get_backend  # noqa: PLC0415
 
-        token = Root.set(self.cfg["root"])
-        try:
-            will=attrdict(
-                data=dict(
-                    up=False,
-                    state="disconnected",
-                    ),
-                topic=P(":R.run.ping.id")/self.id,
-                retain=False,
-                qos=QoS.AT_LEAST_ONCE,
-            )
-            async with (
-                get_backend(self.cfg, name=self.name, will=will) as self.backend,
-                anyio.create_task_group() as self.tg,
-            ):
-                if self._port is not None:
-                    sdr = await self.tg.start(self._connected_port)
-                else:
-                    if self.cfg.client.init_timeout:
-                        # connect to the main server
-                        await self.tg.start(self._run_server_link)
-                    sdr = _Sender(self)
-                sdr.add_sub("cl")
-                sdr.add_sub("d")
-                sdr.add_sub("e")
-                sdr.add_sub("i")
-                try:
-                    self.sdr = sdr
-                    await self.tg.start(self._ping)
+        will = attrdict(
+            data=NotGiven,
+            topic=P(":R") + self._id_path,
+            retain=True,
+            qos=QoS.AT_LEAST_ONCE,
+        )
+        async with (
+            ctx_as(Root, self.cfg["root"]),
+            get_backend(self.cfg, name=self.name, will=will) as self.backend,
+            anyio.create_task_group() as self.tg,
+        ):
+            if self._port is not None:
+                sdr = await self.tg.start(self._connected_port)
+            else:
+                if self.cfg.client.init_timeout:
+                    # connect to the main server
+                    await self.tg.start(self._run_server_link)
+                sdr = LinkSender(self)
+            sdr.add_sub("cl")
+            sdr.add_sub("d")
+            sdr.add_sub("e")
+            sdr.add_sub("i")
+            try:
+                self.sdr = sdr
+                await self.tg.start(self._monitor_id)
+                await self.tg.start(self._send_ping)
+
+                with ctx_as(_the_link, self) if self._common else nullcontext():
                     yield sdr
-                finally:
-                    del self.sdr
-                self.tg.cancel_scope.cancel()
-                await self.backend.send(Root.get()+self._ping_path, data=dict(up=False,state="closed"), retain=False, meta=False)
-        finally:
-            Root.reset(token)
+            finally:
+                del self.sdr
+            await self.backend.send(
+                Root.get() + self._ping_path,
+                data=dict(up=False, state="closed"),
+                retain=False,
+                meta=False,
+            )
+            await self.backend.send(
+                Root.get() + self._id_path,
+                data=NotGiven,
+                retain=True,
+                meta=False,
+            )
+            self.tg.cancel_scope.cancel()
 
     def cancel(self):
         "Stop me"
         self.tg.cancel_scope.cancel()
 
-
     async def _run_server_link(self, *, task_status=anyio.TASK_STATUS_IGNORED):
         """
         This is the manager task for the server link channel.
@@ -710,23 +885,22 @@ class Link(LinkCommon, CtxObj):
         if isinstance(link, dict):
             link = (link,)
 
-        local_ok = srv.data.get("node","localhost")
-        locals = { "localhost", "127.0.0.1", "::1" }
+        local_ok = srv.data.get("node", "localhost")
 
         for remote in link:
             if local_ok:
                 pass
             elif "host" not in remote:
                 continue
-            elif remote["host"] in locals:
+            elif remote["host"] in local_addrs:
                 continue
             try:
                 async with timed_ctx(
                     self.cfg.client.init_timeout, self._connect_one(remote, srv.data)
                 ) as rem:
                     await self._connect_run(rem, task_status=task_status)
-            except EnvironmentError as exc:
-                self.logger.warning("Link failed: %r", remote)
+            except OSError as exc:
+                self.logger.warning("Link failed: %r (%r)", remote, exc)
             except Exception as exc:
                 self.logger.warning("Link failed: %r", remote, exc_info=exc)
 
@@ -757,7 +931,11 @@ class BasicLink(LinkCommon, CtxObj):
             except Exception as exc:
                 if yielded:
                     raise
-                self.logger.warning("Link failed: %r", remote, exc_info=None if isinstance(exc,EnvironmentError) else exc)
+                self.logger.warning(
+                    "Link failed: %r",
+                    remote,
+                    exc_info=None if isinstance(exc, EnvironmentError) else exc,
+                )
                 if err is None:
                     err = exc
             else:
@@ -777,60 +955,84 @@ class Watcher(CtxObj):
     * a MQTT subscription to a subtree of our MoaT-Link hierarchy
     * a MoaT-Link request to enumerate a subtree
     """
-    link:Link=field()
-    path:Path=field()
-    meta:bool=field()
-    subtree:bool=field()
-    state:bool|None=field()
-    age:float|None=field()
-    mark:bool=field()
-    node_cls:type=field()
 
-    _qw=field(init=False,repr=False)
-    _qr=field(init=False,repr=False)
-    _tg=field(init=False,repr=False)
-    _node=field(init=False,default=None)
+    link: Link = field()
+    path: Path = field()
+    meta: bool = field()
+    subtree: bool = field()
+    state: bool | None | NotGiven = field()
+    age: float | None = field()
+    mark: bool = field()
+    node_cls: type = field()
+    min_length: type = field()
+    max_length: type = field()
 
-    _current_done:anyio.Event|None=field(init=False,default=None)
+    _qr = field(init=False, repr=False)
+    _tg = field(init=False, repr=False)
+    _node = field(init=False, default=None)
+
+    _current_done: anyio.Event | None = field(init=False, default=None)
 
     def __attrs_post_init__(self):
-        self._qw,self._qr = anyio.create_memory_object_stream(99)
+        if self.mark and self.state is NotGiven:
+            raise ValueError("MQTT doesn't send a mark. Sorry.")
+        if self.age is not None:
+            self.age = time.time() - self.age
 
-    async def _current(self, *, task_status):
+    def _chk(self, p, m):
+        if self.age and m.timestamp < self.age:
+            return False
+        if self.min_length is not None and len(p) < self.min_length:
+            return False
+        if self.max_length is not None and len(p) > self.max_length:
+            return False
+        return True
+
+    async def _current(self, qw, *, task_status):
         "get the current state from the server"
         if self.subtree:
             pl = PathLongener(())
-            async with self.link.d.walk(self.path) as mon:
+            args = [self.path, self.age, self.min_length, self.max_length]
+            while args[-1] is None:
+                args.pop()
+            async with self.link.d.walk(*args) as mon:
                 task_status.started()
                 async for r in mon:
-                    n,p,d,*m = r
-                    p = pl.long(n,p)
+                    n, p, d, *m = r
+                    p = pl.long(n, p)
                     m = MsgMeta.restore(m)
-                    await self._qw.send((p,d,m))
+                    if self._chk(p, m):
+                        await qw.send((p, d, m))
                 if self.mark:
-                    await self._qw.send(None)
+                    await qw.send(None)
         else:
             try:
                 r = await self.link.d.get(self.path)
-            except (KeyError,ValueError):
+            except (KeyError, ValueError):
                 task_status.started()
             else:
                 task_status.started()
-                p,d,m = Path(), r[0], MsgMeta.restore(r[1:])
-                await self._qw.send((p,d,m))
+                p, d, m = Path(), r[0], MsgMeta.restore(r[1:])
+                if self._chk(p, m):
+                    await qw.send((p, d, m))
             if self.mark:
-                await self._qw.send(None)
+                await qw.send(None)
 
+        await qw.aclose()
         self._current_done.set()
 
-    async def _updates(self, *, task_status):
+    async def _updates(self, qw, *, task_status):
         "get updates from MQTT"
-        plen = 1+len(self.path)  # add the root tag
-        async with self.link.monitor(Root.get()+self.path, subtree=self.subtree, retained=False) as mon:
+        plen = len(self.path)  # ignores the root prefix
+        async with self.link.monitor(
+            Root.get() + self.path, subtree=self.subtree, retained=(self.state is NotGiven)
+        ) as mon:
             task_status.started()
             async for msg in mon:
-                p,d,m = Path.build(msg.topic[plen:]),msg.data,msg.meta
-                await self._qw.send((p,d,m))
+                p, d, m = Path.build(msg.topic[plen:]), msg.data, msg.meta
+                if self._chk(p, m):
+                    await qw.send((p, d, m))
+        await qw.aclose()
 
     @asynccontextmanager
     async def _ctx(self):
@@ -838,17 +1040,17 @@ class Watcher(CtxObj):
             self._node = self.node_cls()
             self._current_done = anyio.Event()
             self._tg = tg
-            self._qw,self._qr = anyio.create_memory_object_stream(10)
+            qw, self._qr = anyio.create_memory_object_stream(10)
             if self.state is not True:
-                await tg.start(self._updates)
-            if self.state is not False:
-                await tg.start(self._current)
+                await tg.start(self._updates, qw.clone())
+            if self.state is True or self.state is None:
+                await tg.start(self._current, qw.clone())
             else:
                 self._current_done.set()
+            await qw.aclose()
             yield self
             tg.cancel_scope.cancel()
-            await self._qw.aclose()
-            self._node=None
+            self._node = None
 
     def __aiter__(self):
         return self
@@ -867,35 +1069,33 @@ class Watcher(CtxObj):
         await self._current_done.wait()
         return self._node
 
-
-    async def _iter(self, *, task_status:anyio.abc.TaskStatus):
+    async def _iter(self, *, task_status: anyio.abc.TaskStatus):
         task_status.started()
-        qr,self._qr = self._qr,None
+        qr, self._qr = self._qr, None
 
         while True:
             try:
                 msg = await qr.receive()
             except anyio.EndOfStream:
                 return
-            p,d,m = msg
-            if self.age is None or m.timestamp+self.age >= time.time():
-                self._node.set(p,d,m)
+            p, d, m = msg
+            self._node.set(p, d, m)
 
     async def __anext__(self):
         while True:
             try:
                 msg = await self._qr.receive()
             except anyio.EndOfStream:
-                raise StopAsyncIteration
+                raise StopAsyncIteration from None
             if msg is None:
                 return None
-            p,d,m = msg
-            if self.age is None or m.timestamp+self.age >= time.time():
-                if self._node.set(p,d,m, force=self._current_done.is_set()):
-                    if self.meta:
-                        return (p,d,m) if self.subtree else (d,m)
-                    else:
-                        return (p,d) if self.subtree else d
+            p, d, m = msg
+            if self._node.set(p, d, m, force=self._current_done.is_set()):
+                if self.meta:
+                    return (p, d, m) if self.subtree else (d, m)
+                else:
+                    return (p, d) if self.subtree else d
+
 
 class Walker:
     """
@@ -905,21 +1105,21 @@ class Walker:
     This differs from `Watcher` by not tracking updates, nor keeping a node
     tree in memory.
     """
-    def __init__(self,mon,meta=False):
-        self.mon=mon
+
+    def __init__(self, mon, meta=False):
+        self.mon = mon
         self.pl = PathLongener()
-        self.meta=meta
+        self.meta = meta
 
     def __aiter__(self):
         return self
 
     async def __anext__(self):
         msg = await anext(self.mon)
-        n,p,d,*m = msg.args
-        p = self.pl.long(n,p)
+        n, p, d, *m = msg.args
+        p = self.pl.long(n, p)
         if self.meta:
             m = MsgMeta.restore(m)
-            return p,d,m
+            return p, d, m
         else:
-            return p,d
-
+            return p, d
