@@ -1,20 +1,18 @@
 """
 Command line for gatewaying
 """
+
 from __future__ import annotations
 
-import datetime
-import time
 import anyio
+import sys
 
 import asyncclick as click
-from moat.util import MsgReader, NotGiven, P, PathLongener, attr_args, yprint, Path
-from moat.util.times import ts2iso, humandelta
 
-from moat.link.client import Link
+from moat.util import NotGiven, P, attr_args, yprint
 from moat.link._data import data_get, node_attr
+from moat.link.client import Link
 from moat.link.meta import MsgMeta
-from moat.link.node import Node
 
 
 @click.group(short_help="Manage gateways.", invoke_without_command=True)
@@ -26,9 +24,9 @@ async def cli(ctx, path):
     """
     obj = ctx.obj
     cfg = obj.cfg["link"]
-    if obj.port is not None:
+    if obj.get("port", None) is not None:
         cfg.client.port = obj.port
-    obj.conn = await ctx.with_async_resource(Link(cfg, name=obj.name))
+    obj.conn = await ctx.with_async_resource(Link(cfg))
     obj.path = path
     if ctx.invoked_subcommand is None:
         await _list(obj)
@@ -40,10 +38,11 @@ async def run(obj):
     """
     Run a gateway setup.
     """
-    from moat.link.gate import run_gate
-    from moat.util.systemd import as_service
+    from moat.link.gate import run_gate  # noqa: PLC0415
+    from moat.util.systemd import as_service  # noqa: PLC0415
+
     async with as_service(obj) as srv:
-        await srv.tg.start(run_gate, obj.cfg, obj.conn, P("gate")+obj.path)
+        await srv.tg.start(run_gate, obj.cfg, obj.conn, P("gate") + obj.path)
         srv.set()
         await anyio.sleep_forever()
 
@@ -56,11 +55,12 @@ async def list_(obj):
     """
     await _list(obj)
 
+
 async def _list(obj):
     if not obj.path:
         seen = False
-        async with obj.conn.d_walk(P("gate"),min_depth=1,max_depth=1) as mon:
-            async for p,d in mon:
+        async with obj.conn.d_walk(P("gate"), min_depth=1, max_depth=1) as mon:
+            async for p, _d in mon:
                 seen = True
                 print(p[-1])
         if not seen and obj.debug:
@@ -71,7 +71,7 @@ async def _list(obj):
     k["recursive"] = True
     k["raw"] = True
     k["empty"] = True
-    await data_get(obj.conn, P("gate")+obj.path, **k)
+    await data_get(obj.conn, P("gate") + obj.path, **k)
 
 
 @cli.command("set", short_help="Add or update a gate entry")
@@ -80,7 +80,7 @@ async def _list(obj):
 @click.option("-D", "--dst", type=P, help="Destination (driver specific)")
 @click.option("-d", "--driver", type=str, help="Driver")
 @click.pass_obj
-async def set_(obj, src,dst,driver, **kw):
+async def set_(obj, src, dst, driver, **kw):
     """
     Add/change a gateway entry.
     """
@@ -93,11 +93,11 @@ async def set_(obj, src,dst,driver, **kw):
         kw["path_"] += ((P("src"), src),)
     if dst:
         kw["path_"] += ((P("dst"), dst),)
-    res = await node_attr(obj, P("gate")+obj.path, **kw)
+    await node_attr(obj, P("gate") + obj.path, **kw)
 
 
-class nstr:
-    def __new__(cls, val):
+class nstr:  # noqa: D101
+    def __new__(cls, val):  # noqa: D102
         if val is NotGiven:
             return val
         return str(val)
@@ -126,13 +126,12 @@ async def delete(obj, before, recursive):
     """
     args = {}
     if recursive:
-        args["rec"]=recursive
+        args["rec"] = recursive
     if before:
-        args["ts"]=before
+        args["ts"] = before
     res = await obj.conn.d.delete(obj.path, **args)
     if obj.meta:
-        res=dict(data=res[0],meta=MsgMeta.restore(res[1:]).repr())
+        res = dict(data=res[0], meta=MsgMeta.restore(res[1:]).repr())
     else:
         res = res[0]
     yprint(res, stream=obj.stdout)
-
