@@ -9,16 +9,16 @@ from logging import getLogger
 from attrs import define, field
 
 from moat.util import NotGiven, Path, PathLongener, PathShortener
+from moat.link.meta import MsgMeta
 from moat.util.exc import ExpKeyError
 
-from ..meta import MsgMeta
-
 from typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
+    from moat.lib.cmd import Key
+
     from collections.abc import Awaitable, Callable, Iterator
     from typing import Any
-    from moat.lib.cmd import Key
 
 logger = getLogger(__name__)
 
@@ -34,7 +34,7 @@ class Node:
     _data: Any = field(init=False, default=NotGiven)
     _meta: MsgMeta | None = field(init=False, default=None)
 
-    _sub: dict[Key,Node] = field(init=False, factory=dict, repr=_keys_repr)  # sub-entries
+    _sub: dict[Key, Node] = field(init=False, factory=dict, repr=_keys_repr)  # sub-entries
 
     def set(self, item: Path, data: Any, meta: MsgMeta, force: bool = False) -> bool | None:
         """Save new data below this node.
@@ -51,11 +51,12 @@ class Node:
                 return False
             if not force and s._data == data:  # noqa:SLF001
                 return None
-        s.set_(item,data,meta)
+        s.set_(item, data, meta)
         return True
 
-    def set_(self, path:Path, data:Any, meta:MsgMeta):
+    def set_(self, path: Path, data: Any, meta: MsgMeta):
         "Low-level node data setter. The (sub)path is not stored by default."
+        path  # noqa:B018
         self._data = data
         self._meta = meta
 
@@ -71,10 +72,10 @@ class Node:
         "return current data, returns NotGiven if empty"
         return self._data
 
-    def keys(self):
+    def keys(self):  # noqa: D102
         return self._sub.keys()
 
-    def items(self):
+    def items(self):  # noqa: D102
         return self._sub.items()
 
     def __bool__(self) -> bool:
@@ -92,7 +93,7 @@ class Node:
         # Iterator that returns a serialization of this node tree.
         ps = PathShortener()
         for p, d, m in self._dump_x_(()):
-            s, p = ps.short(p)
+            s, p = ps.short(p)  # noqa:PLW2901
             yield (s, p, d, *(m.dump() if m is not None else ()))
 
     def _dump_x_(self, path):
@@ -124,7 +125,7 @@ class Node:
             level += len(path)
             path = ()
         for k, v in self._sub.items():
-            it = iter(v._dump(path + (k,), level))
+            it = iter(v._dump(path + (k,), level))  # noqa: SLF001
             try:
                 d = next(it)
             except StopIteration:
@@ -149,18 +150,18 @@ class Node:
             p = pl.long(s, p)
             n = self.get(p)
             if force or n.meta is None or n.meta.timestamp < m.timestamp:
-                n._data = d
-                n._meta = m
+                n._data = d  # noqa: SLF001
+                n._meta = m  # noqa: SLF001
 
     @property
-    def meta(self) -> MsgMeta|None:
+    def meta(self) -> MsgMeta | None:
         "return current metadata"
         return self._meta
 
     @meta.deleter
     def meta(self) -> None:
         "Clear metadata"
-        del self._meta
+        self._meta = None
 
     def __delitem__(self, item) -> None:
         """
@@ -170,7 +171,7 @@ class Node:
         """
         d = self._sub[item]
 
-        if d._sub or d.data is not NotGiven:
+        if d._sub or d.data_ is not NotGiven:  # noqa: SLF001
             raise ValueError(item)
         del self._sub[item]
 
@@ -185,7 +186,7 @@ class Node:
                 try:
                     s = s._sub[k]  # noqa:SLF001
                 except KeyError:
-                    raise ExpKeyError(k)
+                    raise ExpKeyError(k) from None
         else:
             s = self._sub[item]
 
@@ -217,7 +218,7 @@ class Node:
                         raise
                     s = s._add(k)  # noqa:SLF001
                 else:
-                    if create is True and n == len(item) - 1 and s._data is not NotGiven:
+                    if create is True and n == len(item) - 1 and s._data is not NotGiven:  # noqa: SLF001
                         raise KeyError(k)
             return s
 
@@ -228,7 +229,7 @@ class Node:
                 raise
             return self._add(item)
         else:
-            if create is True and res._data is not NotGiven:
+            if create is True and res._data is not NotGiven:  # noqa: SLF001
                 raise KeyError(item)
             return res
 
@@ -264,10 +265,10 @@ class Node:
 
     async def walk(
         self,
-        proc: Callable[[Path, Node], Awaitable[bool|None]],
+        proc: Callable[[Path, Node], Awaitable[bool | None]],
         max_depth: int = 999999,
         min_depth: int = 0,
-        timestamp: int | float = 0,
+        timestamp: float = 0,
         depth_first: bool = False,
         force: bool = False,
     ):
@@ -283,22 +284,23 @@ class Node:
         """
 
         async def _walk(s, p):
-            if depth_first and max_depth > len(p):
-                for k, v in s._sub.items():
+            if depth_first and (max_depth is None or max_depth > len(p)):
+                for k, v in s._sub.items():  # noqa: SLF001
                     await _walk(v, p / k)
 
-            if min_depth <= len(p) and (force or (s.meta is not None and s.meta.timestamp >= timestamp)):
+            if (min_depth is None or min_depth <= len(p)) and (
+                force or (s.meta is not None and s.meta.timestamp >= timestamp)
+            ):
                 if await proc(p, s) is False:
                     return
 
-            if not depth_first and max_depth > len(p):
-                for k, v in list(s._sub.items()):
+            if not depth_first and (max_depth is None or max_depth > len(p)):
+                for k, v in list(s._sub.items()):  # noqa: SLF001
                     await _walk(v, p / k)
 
         await _walk(self, Path())
 
-
-    def search(self, path:Path) -> Node:
+    def search(self, path: Path) -> Node:
         """
         Find the destination node of a path, including wildcards.
         """
@@ -312,8 +314,9 @@ class NodeFinder:
     """A generic object that can walk down a possibly-wildcard-equipped path.
 
     Example: given a path `one.two.three` and a root with subtree `*.three`,
-    `NodeFinder(root).step(one).step(two).step(three)` will return the node
-    at `*.three` (assuming that nothing more specific hangs off `the root`).
+    `NodeFinder(root).step(one).step(two).step(three).result` will return
+    the node at `*.three` (assuming that nothing more specific hangs off
+    the root).
 
     If nothing is found, raises `KeyError`.
     """
@@ -322,14 +325,21 @@ class NodeFinder:
         self.steps = ((src, False),)
 
     def step(self, name, new=False):
+        """
+        Walk a single hierarchy step, observing wildcards.
+
+        @new must not be true; it's here for override compatibility.
+        """
+        if new:
+            raise ValueError("I can't create new nodes.")
         steps = []
-        for node, keep in self.steps:
+        for node, _keep in self.steps:
             if name in node:
                 steps.append((node.get(name), False))
-        for node, keep in self.steps:
+        for node, _keep in self.steps:
             if "+" in node:
                 steps.append((node.get("+"), False))
-        for node, keep in self.steps:
+        for node, _keep in self.steps:
             if "#" in node:
                 steps.append((node.get("#"), True))
         for node, keep in self.steps:
@@ -342,7 +352,6 @@ class NodeFinder:
         self.steps = steps
 
     @property
-    def result(self) -> tuple[Path,Node]:
+    def result(self) -> tuple[Path, Node]:  # noqa: D102
         s = self.steps[0]
         return s[0]
-
